#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)Node.pm	1.25	01/10/16 SMI"
#
# Node staus and info class

package Cluster::Node;
use strict;
use POSIX;
use Cluster::Common;
use Cluster::RunCommand;
use Sun::Solaris::Utils qw(gettext);
use vars qw(@ISA $VERSION);
$VERSION = '1.00';
@ISA = qw(Cluster::Common);

##############################################################################
#
# Class global vars
#
##############################################################################

# File names and location
my $VIEW = '/cgi-bin/node/node_info.pl';

# Cluster objects
my $cmd = new Cluster::RunCommand;

##############################################################################
#
# Class Methods
#
##############################################################################

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    bless ($self, $class);
    return $self;
}

my @status_icons = (
	["Online", "Online", "ok"], # gettext("Online")
	["Offline", "Offline", "critical"] # gettext("Offline")
);

##############################################################################
#
# Updated Cluster Manager Look and Feel functions.
#
# See http://spgweb.eng/hci/behret/sc3x-cm/ for more information.
#
##############################################################################

#
# Print the node status table
#

sub node_status_table ($$) {
	my ($self, $q) = @_;

	# Set the headers for the table. They must be passed in a list of
	# pairs consisting of the header text, and the width of the column
	# expressed in either percentage or absolute pixels.
	my @nodeheaders = (gettext("Name"), "33%",
			   "&nbsp;", 14,
			   gettext("Status"), "67%");

	# Determine the number of columns from the headers
	my $numcols = ($#nodeheaders + 1) / 2;

	# Start the property table
	$q->start_prop_table(gettext("Nodes"), \@nodeheaders,
		    "/images/node_16.gif"); 

	# Get the cluster nodelist
	my @nodelist = $self->get_nodelist();

	# Cycle through the list of nodes and print the status
	for (my $i=0; $i<=$#nodelist; $i++) {

		# Get the node status
		my $nodestate = $self->get_nodestate($nodelist[$i]);

		# Start a table row for the node
		$q->start_prop_tr();

		# Print a table cell for the nodename
		$q->start_prop_td();

		# This function will print an icon and a link. It takes the label
		# text, the alt tag text, the link to point to, and the icon to
		# display before the text.
		$q->table_link_text($nodelist[$i], "Node",
				    "/cgi-bin/node/node_info.pl?node=$nodelist[$i]");

		$q->end_prop_td();

		# Print a table cell for the status
		$q->start_prop_td();

		# This function will print the correctly-colored & styled text for
		# the given status.
		$q->table_status_text($nodestate, \@status_icons);

		# End the table cell and table row
		$q->end_prop_td();
		$q->end_prop_tr();

		# If we're not on our last row, print the line row divider,
		# giving the width of the table, in columns.
		if ($i < $#nodelist) {
			$q->line_row($numcols);
		}
	}

	# End the node status table
	$q->end_prop_table();
}

#
# Get the node properties from scconf -p
#

sub get_scconf_node_properties($$) {
	my ($self, $node) = @_;
	my (@p_list, %p_item, @parts);

	my ($update, @SCCONFPOUT) = $cmd->scconfp_cache();

	for (my $j=0, my $i=0; $j <= $#SCCONFPOUT; $j++) {
		my $line = $SCCONFPOUT[$j];
		if (!($line =~ /Cluster node name:/)) {
			next;
		}
		@parts = split /(\S+)/, $line;
		if ($parts[7] eq $node) {
			do {
				@parts = split /(\:)/, $line;	

				# Remove leading spaces
				$parts[0] =~ s/^ *//;
				# Uppercase each word of the property name
				my @pname = split / /, $parts[0];
				for (my $i=0; $i<=$#pname; $i++) {
					$pname[$i] = ucfirst($pname[$i]);
				}
				$p_list[$i]{'property'} = join(' ', @pname);
				$p_list[$i]{'value'} = $parts[2];

				# Get rid of "<" and ">"
				$p_list[$i]{'value'} =~ tr/\</ /;
				$p_list[$i]{'value'} =~ tr/\>/ /;
				$line = $SCCONFPOUT[++$j]; 
				$i++;
			} while (!($line =~ /Node transport adapter:/) &&
				 !($line =~ /Cluster/));
			last;
		}
	}

	# Fix! Clean out the last element if it's partially empty
	if (!defined $p_list[$#p_list]{'property'} ||
	    !defined $p_list[$#p_list]{'value'} ||
	    $p_list[$#p_list]{'property'} eq "" ||
	    $p_list[$#p_list]{'value'} eq "") {
		pop @p_list;
	}

	# Splice in the node status into the table at offset 1
	my $nodestatus = $self->get_nodestate($node);
	$p_item{'property'} = "Current Status"; # gettext("Current Status")
	$p_item{'value'} = $nodestatus;
	splice @p_list, 1, 1, \%p_item, $p_list[1];

	return @p_list;
}

#
# Get properties of the machine, if it's online.
#

sub get_system_node_properties($$) {
	my ($self, $node) = @_;
	my (@p_list, $location, @output, $currow, $status);

	$status = $self->get_nodestate($node);
	
	if ($status eq "UP") {
		$location = "/cgi-bin/node/get_hardware.pl";
		@output = $self->get_http($node, 
					  $self->get_http_port(),
					  $location);

		for (my $i=0, my $j=0; $i <= $#output; $i = $i + 2, $j++) {
			$p_list[$j]{'property'} = $output[$i];
			$p_list[$j]{'value'} = $output[$i+1];
		}
	}	

	return @p_list;
}

#
# Get all node properties
#

sub get_node_properties($$) {
	my ($self, $node) = @_;
	my @p_list = $self->get_scconf_node_properties($node);
	push @p_list, $self->get_system_node_properties($node);
	return @p_list;
}

#
# Print a table of node-specific properties
#

sub node_info_table ($$) {
    my ($self, $q, $node) = @_;

    # Set the headers for the table. They must be passed in a list of
    # pairs consisting of the header text, and the width of the column
    # expressed in either percentage or absolute pixels.
    my @nodeheaders = (gettext("Property"), "50%",
		       "&nbsp;", "14",
		       gettext("Value"), "50%");

    # Determine the number of columns from the headers
    my $numcols = ($#nodeheaders + 1) / 2;
    
    # Create the title for the table
    my $headertext = $q->img({ SRC => "/images/node_16.gif",
	alt => gettext("Node") });
    $headertext .= "&nbsp;" . sprintf(gettext("Node Properties - %s"), $node);

    # Start the property table
    $q->start_prop_table($headertext, \@nodeheaders);
    
    # Get the cluster nodelist
    my @property_list = $self->get_node_properties($node);

    # If we don't have any properties, print a message
    if ($#property_list == -1) {
	
	# Start a table row and cell with the colspan of the table
	$q->start_prop_tr();
	$q->start_prop_td({ COLSPAN => 3 });
	
	# Print the message in table-label-text style
	$q->start_table_text('table-label-text');
	print gettext("Node data is not available");
	$q->end_table_text();
	
	# End the cell and row
	$q->end_prop_td();
	$q->end_prop_tr();
    }
    
    # Cycle through the list of nodes and print the status
    for (my $i=0; $i<=$#property_list; $i++) {
	 
	if($property_list[$i]{'property'} ne "Node Enabled" && 
	    $property_list[$i]{'property'} ne "Node Reservation Key") {

	    # Start a table row for the node
	    $q->start_prop_tr();

	    # Print a table cell for the nodename
	    $q->start_prop_td();

	    $q->start_table_text('table-label-text');
	    print gettext($property_list[$i]{'property'}) . ":";
	    # These strings come from scconf, so scconf's message file
	    # could be used instead of defining them here
	    ;# gettext("Cluster Node Name");
	    ;# gettext("Current Status");
	    ;# gettext("Node ID");
	    ;# gettext("Node Private Hostname");
	    ;# gettext("Node Quorum Vote Count");
	    ;# gettext("Node Transport Adapters");
	    $q->end_table_text();

	    $q->end_prop_td();

	    # Print a table cell for the status
	    $q->start_prop_td();

	    # This function will print the correctly-colored & styled text for
	    # the given status.
	    if ($property_list[$i]{'property'} eq "Current Status") {
		$q->table_status_text($self->trim($property_list[$i]{'value'}),
		    \@status_icons);
	    } else {
		# Just a plain property; no icons
		$q->table_status_text($self->trim($property_list[$i]{'value'}));
	    }

	    # End the table cell and table row
	    $q->end_prop_td();
	    $q->end_prop_tr();

	    # If we're not on our last row, print the line row divider,
	    # giving the width of the table, in columns.
	    if ($i < $#property_list) {
		    $q->line_row($numcols);
	    }
	}
    }
    
    # End the node status table
    $q->end_prop_table();
}

##############################################################################
#
# CM New GUI Methods
#
##############################################################################

sub get_menu_data() {
    return ("Foo", "/cgi-bin/bar.html");
}
